import Image from "../entities/Image";
import Database from "../objects/Database";
import RepoBase from "./RepoBase";

export default abstract class ImageRepo {
	public static async SelectAll() {
		const dbImage = await Database.Instance.query("SELECT * FROM Image WHERE IsDeleted = 0");
		const images = new Array<Image>();

		for (const row of dbImage) {
			const image = new Image();
			PopulateImageFromDB(image, row);
			images.push(image);
		}

		return images;
	}

	public static async SelectById(id:number) {
		const dbImage = await Database.Instance.query("SELECT * FROM Image WHERE Id = ? LIMIT 1", [id]);
		if (dbImage == null || dbImage.length === 0) {
			return null;
		} else {
			const image = new Image();
			PopulateImageFromDB(image, dbImage[0]);
			return image;
		}
	}

	public static async InsertUpdate(image: Image) {
		if (image.Id === Number.MIN_VALUE) {
			image.Id = (await Database.Instance.query("INSERT Image (UserId, DomainId, FileName, ImageTag, ImageType, Hash, FileSize, CreatedByUserId, CreatedDatetime, LastModifiedByUserId, LastModifiedDatetime, DeletedByUserId, DeletedDatetime, IsDeleted) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) RETURNING Id;", [
				image.UserId, image.DomainId, image.FileName, image.ImageTag, image.Hash, image.FileSize, image.CreatedByUserId, image.CreatedDatetime.getTime(), image.LastModifiedByUserId ?? null, image.LastModifiedDatetime?.getTime() ?? null, image.DeletedByUserId ?? null, image.DeletedDatetime?.getTime() ?? null, Number(image.IsDeleted)
			]))[0]["Id"];
		} else {
			await Database.Instance.query(`UPDATE Image SET UserId = ?, DomainId = ?, FileName = ?, ImageTag = ?, ImageType = ?, Hash = ?, FileSize = ?, CreatedByUserId = ?, CreatedDatetime = ?, LastModifiedByUserId = ?, LastModifiedDatetime = ?, DeletedByUserId = ?, DeletedDatetime = ?, IsDeleted = ? WHERE Id = ?`, [
				image.UserId, image.DomainId, image.FileName, image.ImageTag, image.Hash, image.FileSize, image.CreatedByUserId, image.CreatedDatetime.getTime(), image.LastModifiedByUserId ?? null, image.LastModifiedDatetime?.getTime() ?? null, image.DeletedByUserId ?? null, image.DeletedDatetime?.getTime() ?? null, Number(image.IsDeleted), image.Id
			]);
		}

		return image;
	}
}

function PopulateImageFromDB(image: Image, dbImage: any) {
	image.Id = dbImage.Id;
	image.UserId = dbImage.UserId;
	image.DomainId = dbImage.DomainId;
	image.FileName = dbImage.FileName;
	image.ImageTag = dbImage.ImageTag;
	image.ImageType = dbImage.ImageType;
	image.Hash = dbImage.Hash;
	image.FileSize = dbImage.FileSize;
	image.CreatedByUserId = dbImage.CreatedByUserId;
	image.CreatedDatetime = new Date(dbImage.CreatedDatetime);
	image.LastModifiedByUserId = dbImage.LastModifiedByUserId;
	image.LastModifiedDatetime = RepoBase.convertNullableDatetimeIntToDate(dbImage.LastModifiedDatetime);
	image.DeletedByUserId = dbImage.DeletedByUserId;
	image.DeletedDatetime = RepoBase.convertNullableDatetimeIntToDate(dbImage.DeletedDatetime);
	image.IsDeleted = dbImage.IsDeleted[0] === 1;
}